# Keyword: `auto` 

The `auto` keyword in C++ is used for type inference, allowing the compiler to automatically deduce the type of a variable based on its initializer. This can help make code more concise and easier to read, especially in cases where the type is complex or verbose.

## Example1:  Basic Usage

```cpp
#include <iostream>
#include <vector>

int main() {
    int x = 10;
    auto y = x;  // y is automatically deduced to be of type int

    std::vector<int> vec = {1, 2, 3, 4, 5};
    auto it = vec.begin();  // it is of type std::vector<int>::iterator

    for (auto elem : vec) {  // elem is deduced to be of type int
        std::cout << elem << " ";
    }
    std::cout << std::endl;

    return 0;
}
```

### Explanation

- `auto y = x;` deduces `y` to be of type `int` because `x` is an `int`.
- `auto it = vec.begin();` deduces it to be of type `std::vector<int>::iterator`.
- In the range-based for loop, `elem` is automatically deduced to be the type of the elements in the vector, which is `int`.
- Advantages of Using auto
    - Simplifies Code: auto is especially useful when dealing with complex types, such as iterators, lambdas, or template-generated types.
    - Improves Readability: By reducing the verbosity of type declarations, it can make the code easier to read and maintain.
    - Eases Maintenance: If the type of a variable changes, you only need to update the initialization expression without needing to change the type declaration.

## Example2: Complex Types

Consider a case where you are working with a map, and you want to iterate over its elements:
```cpp
include <iostream>
include <map>

int main() {
    std::map<std::string, int> myMap = {{"apple", 1}, {"banana", 2}, {"cherry", 3}};

    // Using auto to simplify iterator declaration
    for (auto it = myMap.begin(); it != myMap.end(); ++it) {
        std::cout << it->first << ": " << it->second << std::endl;
    }

    return 0;
}
```

### Explanation

- `auto it = myMap.begin();` deduces the type of it to be `std::map<std::string, int>::iterator`, which can be cumbersome to write explicitly.

## Example 3: Auto with Functions

`auto` can also be used in the context of function return types, especially with C++14 and later, where it can be combined with the `decltype` keyword or used in a trailing return type:
```cpp
include <iostream>

auto add(int a, int b) -> int {
    return a + b;
}

int main() {
    auto result = add(5, 10);  // result is deduced to be of type int
    std::cout << "Result: " << result << std::endl;

    return 0;
}
```

### Explanation

- `auto add(int a, int b) -> int` defines a function where the return type is specified after the function arguments using the trailing return type syntax.
- The auto keyword in `auto result = add(5, 10);` deduces the type of result as `int`.

## Example 4: Auto with Lambda Function 

`auto` is often used in conjunction with lambda expressions, where the return type can be complex or unnecessary to specify explicitly
```cpp

include <iostream>
include <vector>
include <algorithm>

int main() {
    std::vector<int> numbers = {1, 2, 3, 4, 5};

    auto square = [](int x) { return x * x; };

    std::vector<int> squares;
    std::transform(numbers.begin(), numbers.end(), std::back_inserter(squares), square);

    for (auto num : squares) {
        std::cout << num << " ";
    }
    std::cout << std::endl;

    return 0;
}
```

### Explanation

- `auto square = [](int x) { return x * x; };` uses auto to deduce the type of the lambda, which returns `int` but doesn’t require explicit return type declaration.
- The use of `auto` in for (`auto num : squares`) simplifies the loop by automatically deducing num as int.

## Summary

- Initialization Required: `auto` requires an initializer to deduce the type. Without an initializer, the type cannot be inferred.
- Const and Reference Qualifiers: `auto` does not automatically deduce `const` or reference types unless specified in the initializer. For example, `auto&` can be used to deduce a reference type.

## Example 5: Usage with reference to const 

```cpp
include <iostream>

int main() {
    int x = 5;
    const int& ref_x = x;

    auto y = ref_x;    // y is an int (copy of x)
    auto& z = ref_x;   // z is a const int& (reference to x)

    std::cout << "y: " << y << std::endl;  // Outputs: y: 5
    std::cout << "z: " << z << std::endl;  // Outputs: z: 5

    return 0;
}
```

### Explanation:

- `int x = 5;` declares an integer `x`.
- `const int& ref_x = x;` creates a constant reference to `x`, meaning `ref_x` can refer to `x` but cannot modify it.
- `auto y = ref_x;` deduces the type of `y` as `int (a copy of x)`, not `const int&`, because `y` is not explicitly declared as a reference.
- `auto& z = ref_x;` deduces `z` as `const int&`, maintaining the reference and `const` qualifier.


## Summary

- `auto` is a powerful tool for type inference in C++, simplifying the code and reducing verbosity.
- It is commonly used with iterators, lambdas, and complex types.
- It requires an initializer and will deduce the type based on that initializer.

